"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addServerDependency = void 0;
const tslib_1 = require("tslib");
const fs_1 = require("fs");
const path = tslib_1.__importStar(require("path"));
var NodeType;
(function (NodeType) {
    NodeType["OTHER"] = "OTHER";
    NodeType["GENESIS_OPENING_BLOCK"] = "GENESIS_OPENING_BLOCK";
    NodeType["DEPENDENCIES_OPENING_BLOCK"] = "DEPENDENCIES_OPENING_BLOCK";
    NodeType["DEPENDENCY"] = "DEPENDENCY";
})(NodeType || (NodeType = {}));
const parseSettings = (contents) => parseGradleStructure(contents);
const parseBuild = (contents) => parseGradleStructure(contents, true);
const parseProperties = (contents, keyToUpsert) => {
    let insertionIndex;
    let updateIndex;
    const lines = contents.split('\n').map((raw, index) => {
        if (raw.includes('authVersion')) {
            insertionIndex = index + 1;
        }
        if (raw.includes(keyToUpsert)) {
            updateIndex = index;
        }
        return {
            raw,
            type: NodeType.OTHER,
        };
    });
    if (updateIndex) {
        lines.splice(updateIndex, 1);
        insertionIndex = updateIndex;
    }
    return { lines, insertionIndex };
};
const insertIntoArray = (arr, item, index) => [
    ...arr.slice(0, index),
    item,
    ...arr.slice(index),
];
const createDepNode = (name, artifact, entryCreator) => ({
    type: NodeType.DEPENDENCY,
    raw: entryCreator(artifact, name),
    meta: {
        dependency: name,
        dependencyVersionProperty: `${name}Version`,
    },
});
const addDependencyToSettings = (contents, name, artifact) => addDependency(contents, name, artifact, createDependencyEntry, parseSettings);
const addOrUpdateDependencyToBuild = (contents, name, artifact) => addDependency(contents, name, artifact, createImplementationEntry, parseBuild);
const addDependency = (contents, name, artifact, entryCreator, parser) => {
    const dependencyEntry = entryCreator(artifact, name);
    if (contents.includes(dependencyEntry)) {
        return contents;
    }
    const { lines, insertionIndex } = parser(contents);
    const dep = createDepNode(name, artifact, entryCreator);
    const linesUpdated = insertIntoArray(lines, dep, insertionIndex);
    const serialized = linesUpdated.map((l) => l.raw).join('\n');
    return serialized;
};
const addOrUpdateDependencyToProperties = (contents, id, version) => {
    const keyToUpsert = `${id}Version=`;
    const { lines, insertionIndex } = parseProperties(contents, keyToUpsert);
    const dep = { raw: `${keyToUpsert}${version}` };
    const linesUpdated = insertIntoArray(lines, dep, insertionIndex);
    const serialized = linesUpdated.map((l) => l.raw).join('\n');
    return serialized;
};
const addServerDependency = ({ directory }, { name, artifact, target }, version) => {
    // PBC being added / updated
    if (!target || target === 'settings') {
        const settingsPath = `${directory}/server/settings.gradle.kts`;
        const settings = (0, fs_1.readFileSync)(settingsPath, 'utf8');
        const updatedSettings = addDependencyToSettings(settings, name, artifact || name);
        (0, fs_1.writeFileSync)(settingsPath, updatedSettings);
        const propertiesPath = `${directory}/server/gradle.properties`;
        const properties = (0, fs_1.readFileSync)(propertiesPath, 'utf8');
        const updatedProperties = addOrUpdateDependencyToProperties(properties, name, version || name);
        (0, fs_1.writeFileSync)(propertiesPath, updatedProperties);
        // Additional artifacts of a PBC being added/updated
    }
    else {
        const buildGradlePath = findAppBuildGradle(`${directory}/server`);
        const buildGradle = (0, fs_1.readFileSync)(buildGradlePath, 'utf8');
        const updatedBuildGradle = addOrUpdateDependencyToBuild(buildGradle, name, artifact);
        (0, fs_1.writeFileSync)(buildGradlePath, updatedBuildGradle);
    }
};
exports.addServerDependency = addServerDependency;
function parseGradleStructure(contents, skipGenesisBlock = false) {
    const REGEX = {
        GENESIS_OPENING_BLOCK: /\s*genesis\s*{\s*/,
        DEPENDENCIES_OPENING_BLOCK: /\s*dependencies\s*{\s*/,
        DEPENDENCY: /\s*dependency\(.*\)\s*/,
    };
    let insideGenesisBlock = skipGenesisBlock;
    let insertionIndex;
    const lines = contents.split('\n').map((raw, index) => {
        let type = NodeType.OTHER;
        const meta = {};
        if (raw.match(REGEX.GENESIS_OPENING_BLOCK)) {
            type = NodeType.DEPENDENCIES_OPENING_BLOCK;
            insideGenesisBlock = true;
        }
        else if (raw.match(REGEX.DEPENDENCIES_OPENING_BLOCK) && insideGenesisBlock) {
            type = NodeType.DEPENDENCIES_OPENING_BLOCK;
            if (!insertionIndex) {
                insertionIndex = index + 1;
            }
        }
        else if (raw.match(REGEX.DEPENDENCY)) {
            type = NodeType.DEPENDENCY;
            const dep = raw.split(':');
            if (dep.length > 1) {
                meta.dependency = dep[1];
                meta.dependencyVersionProperty = dep[2].split('"')[1];
            }
        }
        return {
            raw,
            type,
            meta,
        };
    });
    return {
        lines,
        insertionIndex,
    };
}
function createDependencyEntry(artifact, name) {
    return ('        dependency("global.genesis:' +
        artifact +
        ':${extra.properties["' +
        name +
        'Version"]}")');
}
function createImplementationEntry(artifact, name) {
    return ('    implementation("global.genesis:' + artifact + ':${project.ext["' + name + 'Version"]}")');
}
function findAppBuildGradle(root) {
    const subFolders = (0, fs_1.readdirSync)(root, { withFileTypes: true });
    for (const subFolder of subFolders) {
        if (subFolder.isDirectory()) {
            const subdir = path.join(root, subFolder.name);
            const files = (0, fs_1.readdirSync)(subdir, { withFileTypes: true });
            for (const file of files) {
                if (file.name === 'build.gradle.kts') {
                    return path.join(subdir, file.name);
                }
            }
        }
    }
    throw new Error('Could not find a valid build.gradle.kts under server!');
}
//# sourceMappingURL=server.js.map